@@ -66,6 +66,10 @@ OBJHANDLE VesselContext::GetSurfaceRef() const
 {
     return VESSEL4::GetSurfaceRef();
 }
+void VesselContext::ParseScenarioLineEx(char* line, void* status) const
+{
+    return VESSEL::ParseScenarioLineEx(line, status);
+}
 void VesselContext::clbkSetClassCaps(FILEHANDLE cfg)
 {
     rust_spacecraft_ = std::move(rust_init_fn_(*this));
@@ -88,7 +92,15 @@ int VesselContext::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
 {
     return dyn_vessel_consume_buffered_key(rust_spacecraft_, key, down, kstate);
 }
-
+void VesselContext::clbkLoadStateEx(FILEHANDLE scn, void* status)
+{
+    dyn_vessel_load_state_ex(rust_spacecraft_, scn, status, *this);
+}
+void VesselContext::clbkSaveState(FILEHANDLE scn)
+{
+    VESSEL4::clbkSaveState(scn);
+    dyn_vessel_save_state(rust_spacecraft_, scn);
+}
 // ==============================================================
 // API callback interface used by the init_vessel! macro
 // ==============================================================
