@@ -1,5 +1,7 @@
 // This file is included directly from lib.rs as making it into a module created too many hassles
 use std::os::raw::c_char;
+use std::pin::Pin;
+
 mod vector;
 
 pub use vector::Vector3;
@@ -225,6 +227,7 @@ pub mod ffi {
         type BoxDynVessel = Box<dyn crate::OrbiterVessel>;
         type PtrBoxDynVessel = crate::PtrBoxDynVessel;
 
+        type c_void;
         /// Rust interface to the `VESSELx` abstract classes in Orbiter SDK
         type VesselContext;
 
@@ -261,21 +264,50 @@ pub mod ffi {
         /// Performs a transformation from local vessel to global (ecliptic) coordinates
         fn Local2Global(self: &VesselContext, local: &VECTOR3, global: &mut VECTOR3);
 
+        /// Set the vessel's mean radius
+        ///
+        /// The size should correspond to the vessel's visual representation, for example the mesh used to show the vessel in the simulation window.
+        /// The size parameter is used by Orbiter to determine the camera distance at which the vessel is within visual
+        /// range of the observer camera. It is also used for calculating various physical parameters.
+        /// If SetSize is not called during the vessel setup, the value from the Size entry in the vessel's configuration file
+        /// is used.
         fn SetSize(self: &VesselContext, size: f64);
+        /// Set the vessel's mass-normalised principal moments of inertia (PMI)
         fn SetPMI(self: &VesselContext, pmi: &VECTOR3);
+        /// Set the vessel's empty mass (excluding propellants)
         fn SetEmptyMass(self: &VesselContext, empty_mass: f64);
+        /// Set the camera position for internal (cockpit) view.
+        /// 
+        /// # Arguments
+        /// * `camera_offset` - Camera offset in vessel coordinates
         fn SetCameraOffset(self: &VesselContext, camera_offset: &VECTOR3);
+        /// Defines the three points defining the vessel's ground contact plane
         fn SetTouchdownPoints(self: &VesselContext, pt1: &VECTOR3, pt2: &VECTOR3, pt3: &VECTOR3);
+        /// Reset the force direction of a thruster
         fn SetThrusterDir(self: &VesselContext, th: THRUSTER_HANDLE, dir: &VECTOR3);
+        /// Set thrust level for a thruster
         fn SetThrusterLevel(self: &VesselContext, th: THRUSTER_HANDLE, level: f64);
+        /// Set the thrust level of a thruster for the current time step only
         fn SetThrusterLevel_SingleStep(self: &VesselContext, th: THRUSTER_HANDLE, level: f64);
 
+        /// Load a mesh definition for the vessel from a file
         fn AddMesh(self: &VesselContext, mesh_name: String);
+        /// Load a mesh definition for the vessel from a file displaced by offset `ofs`
+        /// 
+        /// # Arguments
+        /// * `mesh_name` - name of the mesh file (without extension)
+        /// * `ofs` - a displacement vector which describes the offset of the mesh origin against the vessel origin
         fn AddMeshWithOffset(self: &VesselContext, mesh_name: String, ofs: &VECTOR3);
+        /// Add an exhaust render definition for a thruster
         fn AddExhaust(self: &VesselContext, th: THRUSTER_HANDLE, lscale: f64, wscale: f64)
             -> usize;
 
+        /// Create a new propellant resource ("fuel tank")
+        /// 
+        /// Propellant resources are a component of the vessel's propulsion system. They can hold propellants and distribute
+        /// them to connected engines to generate thrust
         fn CreatePropellantResource(self: &VesselContext, mass: f64) -> PROPELLANT_HANDLE;
+        /// Add a logical thruster definition for the vessel
         fn CreateThruster(
             self: &VesselContext,
             pos: &VECTOR3,
@@ -284,24 +316,38 @@ pub mod ffi {
             ph: PROPELLANT_HANDLE,
             isp: f64,
         ) -> THRUSTER_HANDLE;
+        /// Combine thrusters into a logical thruster group
         fn CreateThrusterGroup(
             self: &VesselContext,
             thrusters: &[THRUSTER_HANDLE],
             thgroup_type: THGROUP_TYPE,
         ) -> THGROUP_HANDLE;
 
+        /// Remove all mesh definitions for the vessel
         fn ClearMeshes(self: &VesselContext);
 
+        /// Returns the vessel's name
         fn GetName(self: &VesselContext) -> &str;
+        /// Returns the vessel's current status parameters in a [VesselStatus] structure
         fn GetStatus(self: &VesselContext, status: &mut VESSELSTATUS);
+        /// Returns the current mass of a propellant resource specified by `ph`
         fn GetPropellantMass(self: &VesselContext, ph: PROPELLANT_HANDLE) -> f64;
         /// Get angular velocity (in rad/s) of the spacecraft around its principal axes and store it in `a_vel`
         fn GetAngularVel(self: &VesselContext, a_vel: &mut VECTOR3);
+        /// Returns the vessel's true "airspeed" vector
+        /// 
+        /// This method returns the true airspeed vector in the requested frame of reference. The ground airvector is
+        /// defined as the vessel's velocity vector with respect to the surrounding freestream air flow.
+        /// If the vessel is not within an a planetary atmosphere, the returned vector is equal to the groundspeed vector
         fn GetAirspeedVector(self: &VesselContext, ref_frame: REFFRAME, airspeed: &mut VECTOR3) -> bool;
+        /// Returns thrust force vector in local vessel coordinates
         fn GetThrustVector(self: &VesselContext, thrust_vec: &mut VECTOR3) -> bool;
+        /// Returns the vessel's current total propellant mass
         fn GetTotalPropellantMass(self: &VesselContext) -> f64;
+        /// Returns the mean thrust level for a default thruster group type
         #[rust_name = "GetThrusterGroupLevelByType"]
         fn GetThrusterGroupLevel(self: &VesselContext, thgroup_type: THGROUP_TYPE) -> f64;
+        /// Returns the mean thrust level for a default thruster group specified by `th`
         #[rust_name = "GetThrusterGroupLevel"]
         fn GetThrusterGroupLevel(self: &VesselContext, th: THGROUP_HANDLE) -> f64;
         /// Returns a flag indicating contact with a planetary surface
@@ -321,6 +367,9 @@ pub mod ffi {
         /// Results are returned in the ecliptic frame (ecliptic and equinox of J2000.0). 
         fn GetRelativeVel(self: &VesselContext, href: OBJHANDLE, rel_vel: &mut VECTOR3);
 
+        /// Pass a line read from a scenario file to Orbiter for default processing
+        unsafe fn ParseScenarioLineEx(self: &VesselContext, line: *mut c_char, status: *mut c_void);
+
         /// Print message to lower-left corner of screen. For debugging purposes only!
         fn ODebug(s: String);
 
@@ -339,6 +388,12 @@ pub mod ffi {
         unsafe fn oapiWriteItem_bool(f: FILEHANDLE, item: *mut c_char, val: bool);
         unsafe fn oapiWriteItem_vec(f: FILEHANDLE, item: *mut c_char, val: &VECTOR3);
 
+        unsafe fn oapiWriteScenario_string(scn: FILEHANDLE, item: *mut c_char, val: *mut c_char);
+        unsafe fn oapiWriteScenario_float(scn: FILEHANDLE, item: *mut c_char, val: f64);
+        unsafe fn oapiWriteScenario_int(scn: FILEHANDLE, item: *mut c_char, val: i32);
+        unsafe fn oapiWriteScenario_vec(scn: FILEHANDLE, item: *mut c_char, val: &VECTOR3);
+
+        unsafe fn oapiReadScenario_nextline(scn: FILEHANDLE, line: &mut *mut c_char) -> bool;
         unsafe fn oapiWriteLine(f: FILEHANDLE, line: *mut c_char);
 
         unsafe fn oapiOpenFile(
@@ -372,6 +427,13 @@ pub mod ffi {
             kstate: *mut c_char,
         ) -> i32;
         unsafe fn dyn_vessel_drop_in_place(ptr: PtrBoxDynVessel);
+        unsafe fn dyn_vessel_load_state_ex(
+            vessel: &mut BoxDynVessel,
+            scn: FILEHANDLE,
+            status: *mut c_void,
+            sdk_vessel: Pin<&mut VesselContext>
+        );
+        fn dyn_vessel_save_state(vessel: &mut BoxDynVessel, scn: FILEHANDLE);
     }
 }
 
@@ -423,7 +485,33 @@ unsafe fn dyn_vessel_consume_buffered_key(
     let kstate = crate::KeyStates::from(kstate);
     (**vessel).consume_buffered_key(crate::Key::from(key.0 as u8), down, kstate)
 }
+unsafe fn dyn_vessel_load_state_ex(
+    vessel: &mut BoxDynVessel,
+    scn: FILEHANDLE,
+    status: *mut ffi::c_void,
+    sdk_vessel: Pin<&mut VesselContext>)
+{
+    let mut line: *mut c_char = std::ptr::null_mut();
+    loop {
+        let res = ffi::oapiReadScenario_nextline(scn, &mut line);
+        if !res {
+            break;
+        }
 
+        let line_str = std::ffi::CStr::from_ptr(line).to_string_lossy();
+        if !vessel.on_load_param(&line_str)
+        {
+            sdk_vessel.ParseScenarioLineEx(line, status);
+        }
+    }
+}
+
+fn dyn_vessel_save_state(
+    vessel: &mut BoxDynVessel,
+    scn: FILEHANDLE
+) {
+    vessel.on_save_state(&scn);
+}
 pub use ffi::VesselContext;
 impl std::fmt::Debug for VesselContext {
     fn fmt(&self, fmt: &mut std::fmt::Formatter) -> std::fmt::Result {
